@@ -43,7 +43,7 @@
 #' The \code{as.matrix} methods attempt to coerce their input to a matrix in
 #' adjacency, incidence, or edgelist form.  Edge values (from a stored
 #' attribute) may be used if present. \code{\link[tibble:as_tibble]{as_tibble}}
-#' coerces into an edgelist in \code{\link{tibble}} (a type of
+#' coerces into an edgelist in \code{\link[tibble]{tibble}} (a type of
 #' \code{\link{data.frame}}) form; this can be especially useful if extrecting
 #' a character-type edge attribute.
 #' 
@@ -86,7 +86,7 @@
 #' edglist form?
 #' @param na.rm logical; should missing edges/vertices be included in the
 #' edgelist formats? Ignored if \code{as.sna.edgelist=TRUE}.
-#' @param unit whether a \code{\link{tibble}} of edge or vertex attributes
+#' @param unit whether a \code{\link[tibble]{tibble}} of edge or vertex attributes
 #' should be returned.
 #' @param ...  additional arguments.
 #' @return For \code{as.matrix} methods, an adjacency, incidence, or edgelist
@@ -99,7 +99,7 @@
 #' \code{\link{network.extraction}},\code{\link{as.edgelist}}
 #' @references Butts, C. T.  (2008).  \dQuote{network: a Package for Managing
 #' Relational Data in R.} \emph{Journal of Statistical Software}, 24(2).
-#' \url{https://www.jstatsoft.org/v24/i02/}
+#' \doi{10.18637/jss.v024.i02}
 #' @keywords classes graphs
 #' @examples
 #' 
@@ -272,12 +272,10 @@ as_tibble.network<-function(x,attrnames=(match.arg(unit)=="vertices"),na.rm=TRUE
   df <- df[intersect(c(".tail", ".head", ".eid", attrnames), names(df))]
   for(a in setdiff(attrnames, names(df))) df[[a]] <- rep(list(), nrow(df))
 
-  attr(df,"n")<-network.size(x)
-  attr(df,"vnames")<-network.vertex.names(x)
-  if(is.bipartite(x))
-    attr(df,"bipartite")<-x%n%"bipartite"
-
-  as_tibble(df)
+  structure(as_tibble(df),
+            n = network.size(x),
+            vnames = network.vertex.names(x),
+            bipartite = x %n% "bipartite")
 }
 
 #' @rdname as.matrix.network
@@ -375,7 +373,7 @@ as.network<-function(x,...)
 #' \code{\link{which.matrix.type}}
 #' @references Butts, C. T.  (2008).  \dQuote{network: a Package for Managing
 #' Relational Data in R.} \emph{Journal of Statistical Software}, 24(2).
-#' \url{https://www.jstatsoft.org/v24/i02/}
+#' \doi{10.18637/jss.v024.i02}
 #' @keywords classes graphs
 #' @examples
 #' 
@@ -560,7 +558,7 @@ as.network.matrix<-function(x, matrix.type=NULL,
 #' @seealso \code{\link{as.matrix.network}}, \code{\link{network}}
 #' @references Butts, C. T.  (2008).  \dQuote{network: a Package for Managing
 #' Relational Data in R.} \emph{Journal of Statistical Software}, 24(2).
-#' \url{https://www.jstatsoft.org/v24/i02/}
+#' \doi{10.18637/jss.v024.i02}
 #' @keywords graphs manip
 #' @examples
 #' 
